﻿-------------------------------------------------------------------------------------
-- Derby Lua Script
-- By Bidwars
-------------------------------------------------------------------------------------

local M = {}
math.randomseed(os.time())  --This is used for Random Math.
require('mathlib')

-------------------------------------------------------------------------------------
--CIRCLE INTERSECTION
-------------------------------------------------------------------------------------
local function circleLineIntersection( Ax, Ay, Bx, By, Cx, Cy, R )
	-- compute the euclidean distance between A and B
	local LAB = math.sqrt( (Bx-Ax)*(Bx-Ax)+(By-Ay)*(By-Ay) )

	-- compute the direction vector D from A to B
	local Dx = (Bx-Ax)/LAB
	local Dy = (By-Ay)/LAB

	-- Now the line equation is x = Dx*t + Ax, y = Dy*t + Ay with 0 <= t <= 1.

	-- compute the value t of the closest point to the circle center (Cx, Cy)
	local t = ( Dx*(Cx-Ax) + Dy*(Cy-Ay) )
	
	-- This is the projection of C on the line from A to B.

	-- compute the coordinates of the point E on line and closest to C
	local Ex = t*Dx+Ax
	local Ey = t*Dy+Ay

	-- compute the euclidean distance from E to C
	local LEC = math.sqrt( (Ex-Cx)*(Ex-Cx)+(Ey-Cy)*(Ey-Cy) )

	-- test if the line intersects the circle
	if ( LEC < R ) then
		-- compute distance from t to circle intersection point
		local dt = math.sqrt( R*R - LEC*LEC )
	
		-- compute first intersection point
		local Fx = (t-dt)*Dx + Ax
		local Fy = (t-dt)*Dy + Ay
	
		-- compute second intersection point
		local Gx = (t+dt)*Dx + Ax
		local Gy = (t+dt)*Dy + Ay
		
		return "secant", Fx, Fy, Gx, Gy
		
	-- else test if the line is tangent to circle
	elseif ( LEC == R ) then
		-- tangent point to circle is E
		return "tangent", Ex, Ey
	else
		-- line doesn't touch circle
		return "none"
	end
end
-------------------------------------------------------------------------------------
--RECTANCE LINE INTERSECT
-------------------------------------------------------------------------------------
function getVec2(x_,y_)
    vec2={}
    vec2.x=x_
    vec2.y=y_

    function vec2:dist() --set relative position
        return math.sqrt(vec2.x*vec2.x + vec2.y*vec2.y)
    end

    function vec2:sqrdist() --set relative position
        return vec2.x*vec2.x + vec2.y*vec2.y
    end

    function vec2.mulf(v,f  ) 
        return getVec2(v.x*f , v.y*f)
    end

    function vec2.minus(v1 ,v2  ) 
        return getVec2(v1.x-v2.x , v1.y-v2.y)
    end

    --cross product v × w to be vx wy − vy wx
    function vec2.cross(v1,v2  ) 
        return v1.x*v2.y - v1.y*v2.x
    end
    function vec2.dot(v1,v2  ) 
        return v1.x*v2.x + v1.y*v2.y
    end
    function vec2.add(v1,v2  ) 
        return getVec2(v1.x+v2.x , v1.y+v2.y)
    end

    function vec2.normalize(v1)
        local d = v1:dist() 
        if d> 0 then
            return getVec2(v1.x/d,v1.y/d)
        else
            return getVec2(0,0)
        end
    end

    function vec2.normal(v1)
        return getVec2(-v1.y ,v1.x)
    end

    function vec2.left(v1)
        return getVec2(-v1.y ,v1.x)
    end

    function vec2.right(v1)
        return getVec2(v1.y ,-v1.x)
    end
    return vec2 
end
function vec2findIntersect(p,p1,q,q1)
--http://stackoverflow.com/questions/563198/how-do-you-detect-where-two-line-segments-intersect
	--print("vec2findIntersect")
    local r = vec2.minus(p1 ,p  ) 
    local s = vec2.minus(q1 ,q  ) 

    --t = (q − p) × s / (r × s)
    --u = (q − p) × r / (r × s)

    local q_p = vec2.minus(q,p)
    local q_pdots = vec2.cross(q_p,s)
    local q_pdotr = vec2.cross(q_p,r)
    local rdots = vec2.cross(r,s)

    --If r × s = 0
    if rdots == 0 then
        return false,false,nil
    end

    local t = q_pdots / rdots
    local u = q_pdotr / rdots

    --if 0 ≤ t ≤ 1 and 0 ≤ u ≤ 1, the two line segments meet at the point p + t r = q + u s.
    if ((t>= 0 ) and (t<=1) and (u>= 0 ) and (u<= 1 )) then
        --p + t  *r 
        local ret = vec2.mulf(r,t) 
        ret = vec2.add(p,ret  ) 
        return ret.x,ret.y,ret
    end
    return false,false,nil
end 
function findIntersect(l1p1x,l1p1y, l1p2x,l1p2y, l2p1x,l2p1y, l2p2x,l2p2y)
	--print("findIntersect")
    local p = getVec2(l1p1x,l1p1y)
    local p1 = getVec2(l1p2x,l1p2y)
    local q = getVec2(l2p1x,l2p1y)
    local q1 = getVec2(l2p2x,l2p2y)

    return  vec2findIntersect(p,p1,q,q1)

end
local function findRandomPoint()
	local wp1 = scenario_derby_main.wp1
	local wp2 = scenario_derby_main.wp2
	local pos = {}
	local offset = 20
	if wp2.x > wp1.x then
		wp2.x = wp2.x - offset
		wp1.x = wp1.x + offset
	else
		wp2.x = wp2.x + offset
		wp1.x = wp1.x - offset
	end
	if wp2.y > wp1.y then
		wp2.y = wp2.y - offset
		wp1.y = wp1.y + offset
	else 
		wp2.y = wp2.y + offset
		wp1.y = wp1.y - offset
	end

	pos.x = math.random(wp2.x, wp1.x)
	pos.y = math.random(wp1.y, wp2.y)
	pos.z = wp1.z
	local randomPoint = vec3(pos.x, pos.y, pos.z)
	return randomPoint
end
local function vec2Diff(tpos, tvel, vpos, vposdir, vvel, dt)
	local dirVec = vposdir
	local speed = tvel:length()
	local tposVec = nil 
	if speed > 0 then 
		tposVec = tpos + tvel
	else 
		tposVec = tpos
	end	
	local vposVec = vpos 
	local targetVec = (tposVec - vposVec):normalized()
	--if dirTarget is negitive then the target is behind
	local dirTarget = dirVec:dot(targetVec)
	local dirDiff = math.asin(dirVec.x * targetVec.y - dirVec.y * targetVec.x)
	return dirDiff, dirTarget, dirVec
end

M.findRandomPoint = findRandomPoint
M.findIntersect = findIntersect
M.vec2Diff = vec2Diff
return M